import os
import pickle

import numpy as np
import torch
from PIL import Image, ImageDraw

from navsim.visualization.private import view_points


def main():
    path = "/mnt/g/cvpr_demo/data.pkl"
    demo_dir = "/mnt/f/e2e/navsim_ours/debug/cvpr_demo"
    result_name = 'ego_2048x512'
    results = pickle.load(open(f'{demo_dir}/{result_name}.pkl', 'rb'))
    vis_dir = result_name

    with open(path, 'rb') as fp:
        datas = pickle.load(fp)[::3]
        token = 0
        for data, curr_traj in zip(datas, results):

            cams = data["cameras"]
            # numpy array
            f0 = cams['CAM_F0']['image']
            f0_meta = cams['CAM_F0']['metadata']
            # cam_intrin = f0_meta['intrinsic']
            # extrinsic = f0_meta['extrinsic']
            # cam2lidar_rot = extrinsic[:3, :3]
            # cam2lidar_tran = extrinsic[:3, 3]
            cam2lidar_rot = np.array([[-3.03113239e-03, -1.97862953e-02, 9.99799637e-01],
                                      [-9.99995318e-01, -3.59686629e-04, -3.03884394e-03],
                                      [4.19742025e-04, -9.99804167e-01, -1.97851124e-02]])
            cam2lidar_tran = np.array([1.62402501, -0.00555072, 1.53312061])
            cam_intrin = np.array([[1.545e+03, 0.000e+00, 9.600e+02],
                                   [0.000e+00, 1.545e+03, 5.600e+02],
                                   [0.000e+00, 0.000e+00, 1.000e+00]])
            coordinates = np.zeros((3, 40))

            coordinates[0] = curr_traj[0, :, 0]
            coordinates[1] = curr_traj[0, :, 1]
            coordinates[2] = 0.0

            lidar2cam_rot = np.linalg.inv(cam2lidar_rot)
            coordinates -= cam2lidar_tran.reshape(-1, 1)
            coordinates = np.dot(lidar2cam_rot, coordinates)
            coordinates = np.dot(cam_intrin, coordinates)
            heights = coordinates[2, :]
            points = view_points(coordinates[:3, :], np.eye(3), normalize=True)
            points[2, :] = heights

            mask = np.ones(points.shape[1], dtype=bool)  # type: ignore
            canvas_size = (1080, 1920)
            mask = np.logical_and(mask, points[0, :] < canvas_size[1] - 1)
            mask = np.logical_and(mask, points[0, :] > 0)
            mask = np.logical_and(mask, points[1, :] < canvas_size[0] - 1)
            mask = np.logical_and(mask, points[1, :] > 0)

            points = points[:, mask]
            depth = heights[mask]

            points = np.int16(np.round(points[:2, :]))
            depth = np.int16(np.round(depth))
            overlay_img = Image.new("RGBA", (canvas_size[1], canvas_size[0]), (255, 255, 255, 0))
            draw = ImageDraw.Draw(overlay_img)
            # Populate canvas, use maximum color_value for each bin
            depth_canvas = np.zeros(canvas_size, dtype=np.int16)
            depth_canvas = np.zeros(canvas_size, dtype=np.int16)
            for (col, row), d in zip(points.T, depth):
                depth_canvas[row, col] = d

            depth_canvas = torch.from_numpy(depth_canvas)

            inds = (depth_canvas > 0).nonzero()
            for ind in inds:
                y, x = ind
                x, y = x.item(), y.item()
                r = 5
                draw.ellipse((x - r, y - r, x + r, y + r), fill=(255, 0, 0, 255))

            img = Image.fromarray(f0.astype('uint8'), 'RGB').convert('RGBA')
            final = Image.alpha_composite(img, overlay_img).convert('RGB')

            os.makedirs(f'{demo_dir}/{vis_dir}', exist_ok=True)
            overlay_img.save(f'{demo_dir}/{vis_dir}/{token}-waypoints.png')
            token += 3


if __name__ == "__main__":
    main()
