import os
import pickle
from io import BytesIO

import imageio
import matplotlib.pyplot as plt
import torch
from PIL import Image

# T = 60
# key_frame = 19

T = 50
key_frame = 9
os.makedirs('/mnt/f/e2e/navsim_ours/debug/plantf/vis', exist_ok=True)

def create_gif(traj):
    images = []
    for t in range(T):
        fig, ax = plt.subplots()
        for i in range(traj.shape[1]):  # Plot N points
            x, y = traj[t, i]
            ax.scatter(x, y, s=30 if i == 0 else 10)  # Plotting points as red circles
        ax.set_xlim(-80, 80)  # Adjust the limits as needed
        ax.set_ylim(-80, 80)  # Adjust the limits as needed
        ax.set_title(f'Original Frame {t + 1}')
        ax.set_aspect('equal')
        ax.axis('off')  # Turn off axis
        plt.tight_layout()

        # Save the plot as an image
        buffer = BytesIO()
        plt.savefig(buffer, format='png')
        buffer.seek(0)
        image = Image.open(buffer)
        plt.close()
        images.append(image)
    return images

def create_pred_gif(traj, model_traj):
    images = []
    for t in range(T):
        fig, ax = plt.subplots()
        for i in range(traj.shape[1]):  # Plot N points
            if t >= (key_frame + 1) and i == 0:
                x, y = traj[key_frame, i] + model_traj[t - (key_frame + 1), :2]
                ax.scatter(x, y, s=30)  # Plotting ego pred points
            else:
                x, y = traj[t, i]
                ax.scatter(x, y, s=30 if i == 0 else 10)  # Plotting points as red circles
        ax.set_xlim(-80, 80)  # Adjust the limits as needed
        ax.set_ylim(-80, 80)  # Adjust the limits as needed
        ax.set_title(f'Pred Frame {t + 1}')
        ax.set_aspect('equal')
        ax.axis('off')  # Turn off axis
        plt.tight_layout()

        # Save the plot as an image
        buffer = BytesIO()
        plt.savefig(buffer, format='png')
        buffer.seek(0)
        image = Image.open(buffer)
        plt.close()
        images.append(image)
    return images


if __name__ == '__main__':
    tokens = ['2edb77f22389561d', 'afbcb815d8375374']
    preds = pickle.load(open('/mnt/f/e2e/navsim_ours/debug/plantf/epoch19.pkl', 'rb'))
    for token in tokens:
        traj = torch.load(f'/mnt/f/e2e/navsim_ours/debug/plantf/lctgen/{token}.pth')['traj']
        ori_imgs = create_gif(
            traj,
        )
        pred_imgs = create_pred_gif(
            traj,
            preds[token]['trajectory'].poses,
        )
        final_imgs = []
        for ori_img, pred_img in zip(ori_imgs, pred_imgs):
            new_image = Image.new('RGB', (ori_img.width + pred_img.width, ori_img.height))
            new_image.paste(ori_img, (0, 0))
            new_image.paste(pred_img, (ori_img.width, 0))
            final_imgs.append(new_image)
        imageio.mimsave(f'/mnt/f/e2e/navsim_ours/debug/plantf/vis/{token}.gif', final_imgs, loop=0, duration=0.1)

